from allennlp_models.syntax.srl.srl_eval_scorer import SrlEvalScorer
from allennlp_models.syntax.srl.srl_bert_model import SrlBert
from allennlp_models.syntax.srl.srl_model import SemanticRoleLabeler
from allennlp_models.syntax.srl.srl_predictor import SemanticRoleLabelerPredictor
from allennlp_models.syntax.srl.srl_reader import SrlReader
from allennlp_models.syntax.srl.openie_predictor import OpenIePredictor
