from allennlp_models.syntax.constituency_parser.constituency_parser_model import (
    SpanConstituencyParser,
)
from allennlp_models.syntax.constituency_parser.constituency_parser_predictor import (
    ConstituencyParserPredictor,
)
from allennlp_models.syntax.constituency_parser.penn_tree_bank import (
    PennTreeBankConstituencySpanDatasetReader,
)
