from allennlp_models.syntax.semantic_dependencies.semantic_dependencies_reader import (
    SemanticDependenciesDatasetReader,
)
from allennlp_models.syntax.semantic_dependencies.graph_parser_model import GraphParser
