from allennlp_models.syntax.biaffine_dependency_parser.biaffine_dependency_parser import (
    BiaffineDependencyParser,
)
from allennlp_models.syntax.biaffine_dependency_parser.biaffine_dependency_parser_predictor import (
    BiaffineDependencyParserPredictor,
)
from allennlp_models.syntax.biaffine_dependency_parser.universal_dependencies_reader import (
    UniversalDependenciesDatasetReader,
)
